"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var lodash_1 = require("lodash");
var path_1 = require("path");
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var assert_1 = require("../utils/assert");
var ast_utils_1 = require("../utils/ast.utils");
var file_utils_1 = require("../utils/file.utils");
var rx_file_1 = require("../utils/rx.file");
var semver_utils_1 = require("../utils/semver.utils");
var constants_1 = require("./../constants");
var options_1 = require("./../options");
var rx_walk_1 = require("./../utils/rx.walk");
var abstract_command_1 = require("./abstract.command");
var help_1 = require("./help");
function _modifySource(aCfg, aFile, aLogger) {
    // process the ng-module imports
    var ngModules = aCfg['ngModules'] || {};
    // iterate
    var file = aFile;
    lodash_1.forEach(ngModules, function (fromLocation, moduleName) {
        // log this
        aLogger.next("Importing module [" + moduleName + "] from [" + fromLocation + "] ...");
        // add the module
        file = ast_utils_1.importModule(moduleName, fromLocation, file);
    });
    // ok
    return file;
}
function _extractConfig(aPkg) {
    // root config
    var root = aPkg['wchConfig'] || {};
    // install
    var install = root['install'] || {};
    // ok
    return install;
}
function _rewriteModule(aModuleFile, aLibPkg, aLogger) {
    // log this
    aLogger.next("Rewriting module [" + aModuleFile + "] ...");
    // load the file
    var rxModule = ast_utils_1.rxReadTypescriptFile(aModuleFile);
    var cfg = _extractConfig(aLibPkg);
    // update
    var rxUpdated = rxModule.pipe(operators_1.map(function (mod) { return _modifySource(cfg, mod, aLogger); }));
    // write back
    return rxUpdated.pipe(operators_1.mergeMap(function (mod) { return rx_file_1.rxWriteTextFile(aModuleFile, mod.source); }));
}
function _updatePackageFile(aDstPkg, aLibPkg) {
    // update
    var dstDeps = aDstPkg.dependencies || (aDstPkg.dependencies = {});
    // update
    dstDeps[aLibPkg.name] = "^" + aLibPkg.version;
    // ok
    return aDstPkg;
}
function _rewritePackageJson(aPkgDir, aLibPkg, aLogger) {
    // add the library
    var pkgFile = path_1.join(aPkgDir, 'package.json');
    // log this
    aLogger.next("Rewriting package " + pkgFile + " ...");
    // read, transform, write
    return rx_walk_1.readFile(pkgFile).pipe(operators_1.map(function (data) { return JSON.parse(data); }), operators_1.map(function (pkg) { return _updatePackageFile(pkg, aLibPkg); }), operators_1.map(function (pkg) { return JSON.stringify(pkg, undefined, 2); }), operators_1.mergeMap(function (data) { return rx_file_1.rxWriteTextFile(pkgFile, data); }));
}
var AddLibraryCommand = /** @class */ (function (_super) {
    __extends(AddLibraryCommand, _super);
    function AddLibraryCommand(options) {
        return _super.call(this, options) || this;
    }
    AddLibraryCommand.prototype.printHelp = function () {
        // log the command line
        console.log(AddLibraryCommand.COMMAND, AddLibraryCommand.TYPE, "--" + constants_1.OPTION_NAME + " <NAME>", "[--" + constants_1.OPTION_SOURCE + " <SOURCE>]", "[--" + constants_1.OPTION_PKG_DIR + " <PKG>]", "[--" + constants_1.OPTION_MODULE + " <MODULE>]", "[--" + constants_1.OPTION_URL + " <URL>]", "[--" + constants_1.OPTION_VERBOSE + "]");
        // help on the options
        console.log("--" + constants_1.OPTION_NAME + " <NAME>", ':', help_1.HELP_LIBRARY_NAME);
        console.log("--" + constants_1.OPTION_SOURCE + " <SOURCE>", ':', help_1.HELP_PROJECT_SOURCE);
        console.log("--" + constants_1.OPTION_PKG_DIR + " <PKG>", ':', help_1.HELP_PKG_DIR);
        console.log("--" + constants_1.OPTION_MODULE + " <MODULE>", ':', help_1.HELP_MODULE_FILE);
        console.log("--" + constants_1.OPTION_URL + " <URL>", ':', help_1.HELP_REGISTRY_URL);
        console.log("--" + constants_1.OPTION_VERBOSE, ':', help_1.HELP_VERBOSE);
    };
    AddLibraryCommand.prototype.process = function () {
        // logger
        var logger = this.getLogger();
        // data directory
        var opts = this.getOptions();
        // module file
        var srcDirOpt = options_1.optNormalizeDir(file_utils_1.firstOf(opts.src) || path_1.join(this.getAppDir(), 'app'));
        var moduleFile = opts.module || path_1.join(srcDirOpt, constants_1.APP_MODULE_FILE);
        // package directory
        var pkgDir = this.getPackageDir();
        // registry URL
        var regUrl = opts.url || 'https://registry.npmjs.org/';
        // name of the library
        assert_1.assertParameter(opts.name, constants_1.OPTION_NAME);
        var name = opts.name;
        // parse the name into a reference
        var ref = semver_utils_1.parseReference(name);
        // validate the directory
        var rxSrcDir = this.rxMkdirp(srcDirOpt);
        // log this
        logger.next("Library: " + ref.name + ", version: " + ref.spec + ", ngModule: " + moduleFile + ", registry: " + regUrl);
        // load the package
        var rxLibraryPkg = semver_utils_1.getPackageJson(regUrl, ref).pipe(operators_1.shareReplay());
        // rewrite package
        var rxRewritePackage = rxLibraryPkg.pipe(operators_1.mergeMap(function (libPkg) { return _rewritePackageJson(pkgDir, libPkg, logger); }));
        // rewrite module
        var rxRewriteModule = rxLibraryPkg.pipe(operators_1.mergeMap(function (libPkg) { return _rewriteModule(moduleFile, libPkg, logger); }));
        // done
        var rxTotal = rxjs_1.merge(rxRewritePackage, rxRewriteModule);
        // ok
        return rxTotal;
    };
    AddLibraryCommand.TYPE = 'library';
    AddLibraryCommand.COMMAND = constants_1.CMD_ADD;
    return AddLibraryCommand;
}(abstract_command_1.AbstractCommand));
exports.AddLibraryCommand = AddLibraryCommand;
