"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
/* Copyright IBM Corp. 2017 */
/*-------------------- Imports, consts, helpers -------------------*/
var escapeStringRegexp = require("escape-string-regexp");
var graceful_fs_1 = require("graceful-fs");
var lodash_1 = require("lodash");
var path_1 = require("path");
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
// import { LayoutClass, LayoutRegistry } from '../utils/layout.reg';
var rx_copy_1 = require("../utils/rx.copy");
var constants_1 = require("./../constants");
var file_utils_1 = require("../utils/file.utils");
var options_1 = require("./../options");
var json_1 = require("./../utils/json");
var rx_file_1 = require("./../utils/rx.file");
var rx_hbs_1 = require("./../utils/rx.hbs");
var rx_lines_1 = require("./../utils/rx.lines");
var rx_tidy_1 = require("./../utils/rx.tidy");
var rx_walk_1 = require("./../utils/rx.walk");
var transliteration_1 = require("./../utils/transliteration");
var wchtools_1 = require("./../utils/wchtools");
var abstract_command_1 = require("./abstract.command");
var help_1 = require("./help");
var ASSETS_BASE = path_1.join(__dirname, '..', '..', '..', 'assets');
function _camelCase(aValue) {
    return transliteration_1.camelCase(aValue);
}
function _plural(aKey) {
    return aKey + 's';
}
function _toComment(aValue) {
    return JSON.stringify(json_1.canonicalizeJSON(aValue), undefined, 2)
        .split('\n')
        .map(function (line) { return '     * ' + line; })
        .join('\n');
}
var SINGLE_ELEMENT_TYPES = {};
SINGLE_ELEMENT_TYPES['text'] = 'SingleTextElement';
SINGLE_ELEMENT_TYPES['number'] = 'SingleNumberElement';
SINGLE_ELEMENT_TYPES['toggle'] = 'SingleToggleElement';
SINGLE_ELEMENT_TYPES['formattedtext'] = 'SingleFormattedTextElement';
SINGLE_ELEMENT_TYPES['link'] = 'SingleLinkElement';
SINGLE_ELEMENT_TYPES['datetime'] = 'SingleDateElement';
SINGLE_ELEMENT_TYPES['file'] = 'SingleFileElement';
SINGLE_ELEMENT_TYPES['video'] = 'SingleVideoElement';
SINGLE_ELEMENT_TYPES['image'] = 'SingleImageElement';
SINGLE_ELEMENT_TYPES['optionselection'] = 'SingleOptionSelectionElement';
SINGLE_ELEMENT_TYPES['reference'] = 'SingleReferenceElement';
SINGLE_ELEMENT_TYPES['category'] = 'CategoryElement';
SINGLE_ELEMENT_TYPES['location'] = 'LocationElement';
function _getElementValidator(aType) {
    var type = SINGLE_ELEMENT_TYPES[aType];
    return ('is' +
        (type && type.startsWith('Single') ? type.substr('Single'.length) : type));
}
var MULTI_ELEMENT_TYPES = {};
MULTI_ELEMENT_TYPES['text'] = 'MultiTextElement';
MULTI_ELEMENT_TYPES['number'] = 'MultiNumberElement';
MULTI_ELEMENT_TYPES['toggle'] = 'MultiToggleElement';
MULTI_ELEMENT_TYPES['formattedtext'] = 'MultiFormattedTextElement';
MULTI_ELEMENT_TYPES['link'] = 'MultiLinkElement';
MULTI_ELEMENT_TYPES['datetime'] = 'MultiDateElement';
MULTI_ELEMENT_TYPES['file'] = 'MultiFileElement';
MULTI_ELEMENT_TYPES['video'] = 'MultiVideoElement';
MULTI_ELEMENT_TYPES['image'] = 'MultiImageElement';
MULTI_ELEMENT_TYPES['optionselection'] = 'MultiOptionSelectionElement';
MULTI_ELEMENT_TYPES['reference'] = 'MultiReferenceElement';
var TYPES = {};
TYPES['text'] = 'string';
TYPES['number'] = 'number';
TYPES['toggle'] = 'boolean';
TYPES['formattedtext'] = 'string';
TYPES['link'] = 'Link';
TYPES['datetime'] = 'Date';
TYPES['file'] = 'File';
TYPES['video'] = 'Video';
TYPES['image'] = 'Image';
TYPES['optionselection'] = 'OptionSelection';
TYPES['reference'] = 'RenderingContext';
TYPES['category'] = 'Category';
TYPES['location'] = 'Location';
var AUGMENTED_TYPES = {};
AUGMENTED_TYPES['Image'] = 'SingleImageElement';
AUGMENTED_TYPES['Video'] = 'SingleVideoElement';
AUGMENTED_TYPES['File'] = 'SingleFileElement';
AUGMENTED_TYPES['Link'] = 'SingleLinkElement';
AUGMENTED_TYPES['Category'] = 'CategoryElement';
AUGMENTED_TYPES['Location'] = 'LocationElement';
var DEFAULT_VALUES = {};
DEFAULT_VALUES['text'] = "''";
DEFAULT_VALUES['number'] = '0';
DEFAULT_VALUES['toggle'] = 'false';
DEFAULT_VALUES['formattedtext'] = "''";
var BUILT_IN_TYPES = ['string', 'number', 'boolean', 'Date'];
// main driver function
// calls _getTypeFromMapping() with filter -- get corresponding layout info
// calls _createLayout() -- creates the .jsx and .scss files
// calls _createOverview() -- creates/udpates the app.jsx and layouts/index.jsx files
function _createLayouts(aTools, aSource, aTemplate, aMatcher, aOptions, aLayoutReg, aLogger) {
    // log this
    aLogger.next("createLayouts [" + Object.keys(aTools.layoutMappings) + "]");
    // iterate
    function _isNotNil(aMapping) {
        return !!aMapping;
    }
    function _getTypeFromMapping(aMapping, aTools) {
        // get the ID
        if (aMapping.type && aMapping.type.id) {
            // get the type
            return aTools.types[aMapping.type.id];
        }
    }
    var layouts = rxjs_1.of.apply(void 0, Object.keys(aTools.layoutMappings)).pipe(operators_1.map(function (id) { return aTools.layoutMappings[id]; }), operators_1.filter(_isNotNil), operators_1.filter(function (mapping) { return aMatcher(_getTypeFromMapping(mapping, aTools)); }), operators_1.mergeMap(function (mapping) {
        return _createLayout(mapping, aTools, aTemplate, aOptions, aLayoutReg, aLogger);
    }));
    var overview = _createOverview(aSource, aTemplate, aOptions, aLogger);
    // concat
    return rxjs_1.concat(layouts, overview).pipe(
    // return concat(overview).pipe(
    operators_1.tap(lodash_1.noop, undefined, function () { return aLogger.next('createLayouts done.'); }));
}
// main layout creation function
// calls _getLayouts() -- essentially gets a list of all the layouts we need to create
// calls _createLayoutComponent -- actually generates the .jsx/.scss files for a layout
function _createLayout(aMapping, aTools, aTemplate, aOptions, aLayoutReg, aLogger) {
    // decode the type
    var typeId = aMapping.type.id;
    var type = aTools.types[typeId];
    // check
    if (!type) {
        return rxjs_1.empty();
    }
    /**
     * Get all layouts for a particular layout mapping
     */
    function _getLayouts(aMapping, aTools) {
        var mappings = aMapping.mappings;
        return rxjs_1.Observable.create(function (observer) {
            mappings.forEach(function (mapping) {
                var layouts = mapping.layouts;
                layouts.forEach(function (layout) { return observer.next(aTools.layouts[layout.id]); });
            });
            // done
            observer.complete();
        });
    }
    // check the layouts and create from templates if everything is fine
    return _getLayouts(aMapping, aTools).pipe(operators_1.filter(function (layout) { return layout != null && layout.templateType === 'angular'; }), operators_1.distinct(function (layout) { return layout.id; }), 
    // is name unique??
    operators_1.distinct(function (layout) { return layout.name; }), 
    // resolve the layout class
    operators_1.mergeMap(function (layout) { return aLayoutReg.findLayoutClass(layout, type); }), 
    // produce the layout
    operators_1.mergeMap(function (clz) {
        return _createLayoutComponent(clz, aTools, aTemplate, aOptions, aLayoutReg, aLogger);
    }));
}
// generate the .jsx + .scss files for a layout
function _createLayoutComponent(aClass, aTools, aTemplate, aOptions, aLayoutReg, aLogger) {
    // cycle check
    if (!aLayoutReg.registerLayout(aClass.layout)) {
        // do not create a layout, twice
        return rxjs_1.empty();
    }
    // override flag
    var bOverride = !!aOptions.override;
    // jsx
    /**
     * Constructs the context to generate a layout sample
     */
    function _createLayoutJsxContext(aClass, aTools, aOptions) {
        // elements
        var el = lodash_1.cloneDeep(aClass.type.elements);
        // iterate and add the propery name
        lodash_1.forEach(el, function (e) {
            e.prop = _camelCase(e.key);
            e.onprop = _camelCase('on ' + e.key);
        });
        return {
            // VALUES PASSED INTO HBS TEMPLATE
            className: aClass.layoutClass,
            fileName: aClass.layoutRef,
            editable: !!aOptions.editable || false,
            elements: el
        };
    }
    var jsxCtx = _createLayoutJsxContext(aClass, aTools, aOptions);
    var jsxContent = aTemplate.layoutJsx.pipe(operators_1.mergeMap(function (tmp) { return rx_tidy_1.rxTidy(tmp(jsxCtx).replace(/^\s*[\r\n]/gm, '')); }));
    var dstJsxFile = path_1.join(aClass.jsxFolder, aClass.layoutTemplateFile);
    var jsxFile = jsxContent.pipe(operators_1.mergeMap(function (data) { return rx_file_1.writeTextFileSafe(dstJsxFile, data, bOverride); }));
    // css
    function _createLayoutCssContext(aClass, aTools) {
        return {
            className: aClass.layoutClass
        };
    }
    var cssCtx = _createLayoutCssContext(aClass, aTools);
    var cssContent = aTemplate.layoutCss.pipe(operators_1.map(function (tmp) { return tmp(cssCtx); }));
    var dstCssFile = path_1.join(aClass.cssFolder, aClass.layoutStyleFile);
    var cssFile = cssContent.pipe(operators_1.mergeMap(function (data) { return rx_file_1.writeTextFileSafe(dstCssFile, data, bOverride); }));
    var files = [jsxFile, cssFile];
    // merge
    return rxjs_1.merge.apply(void 0, files);
}
/* generating app.jsx and layouts/index.jsx */
function _createOverview(aSource, aTemplate, aOptions, aLogger) {
    // done
    return rxjs_1.defer(function () {
        var className = /class\s+(\w+)\s+/;
        function _stripComment(aLine) {
            // replace all after "//"
            var idx = aLine.indexOf('//');
            var line = idx >= 0 ? aLine.substr(0, idx) : aLine;
            // replace /* */ like comments
            return line.replace(/(\/\*.*?\*\/)/g, '');
        }
        function _stripSuffix(aValue) {
            var idx = aValue.lastIndexOf('.');
            if (idx >= 0) {
                return aValue.substr(0, idx);
            }
            return aValue;
        }
        // read the class names of these files
        function _extractClass(aFile) {
            // match by line
            var rs = graceful_fs_1.createReadStream(aFile.absPath, 'utf-8');
            // parse the lines
            return rx_lines_1.rxReadLines(rs).pipe(operators_1.map(_stripComment), 
            // skipWhile(line => !layoutComponent.test(line)),
            // skipWhile(line => !component.test(line)),
            operators_1.filter(function (line) { return className.test(line); }), operators_1.map(function (line) { return ({
                root: aFile.root,
                path: './' + _stripSuffix(lodash_1.replace(aFile.relPath, /\\/g, '/')),
                class: className.exec(line)[1],
                abs: aFile.absPath,
                fileName: _stripSuffix(lodash_1.replace(aFile.relPath, /\\/g, '/')).split('/').pop(),
                className: className.exec(line)[1],
                // get relative location for import path
                importPath: ('./' + _stripSuffix(lodash_1.replace(aFile.relPath, /\\/g, '/'))),
                registrationPath: ('./' + LAYOUTS_PATH + '/' + _stripSuffix(lodash_1.replace(aFile.relPath, /\\/g, '/'))),
                // add '-layout' to the layout selector if it doesn't end with list lol
                layoutSelector: transliteration_1.kebabCase(className.exec(line)[1])
                    + (transliteration_1.kebabCase(className.exec(line)[1]).endsWith('list') ? '' : '-layout'),
            }); }));
        }
        function _isRelative(aPath) {
            return !!aPath && (aPath.startsWith('./') || aPath.startsWith('../'));
        }
        function _distinctClassDescriptor(aDesc) {
            return aDesc.abs;
        }
        // main helper function
        function _doExtractBaseClass(aConfig, aCycle) {
            // test if we know this
            if (aCycle[aConfig.abs]) {
                return rxjs_1.empty();
            }
            aCycle[aConfig.abs] = aConfig;
            // match by line
            var rs = graceful_fs_1.createReadStream(aConfig.abs, 'utf-8');
            var parsedPath = path_1.parse(aConfig.abs);
            // parse the lines
            var rxDoc = rx_lines_1.rxReadLines(rs).pipe(operators_1.toArray(), operators_1.map(function (lines) { return lines.join('\n'); }), operators_1.shareReplay());
            // find the base classes
            var rgBaseClasses = new RegExp("class\\s+" + escapeStringRegexp(aConfig.class) + "\\s+extends\\s+([^{]+)", 'g');
            var rxBaseClasses = rxDoc.pipe(operators_1.map(function (doc) { return rgBaseClasses.exec(doc); }), operators_1.filter(function (res) { return !!res && res.length > 1; }), operators_1.map(function (res) { return res[1]; }), operators_1.mergeMap(function (res) { return rxjs_1.from(res.split(',').map(function (cls) { return cls.trim(); })); }), operators_1.mergeMap(function (cls) {
                // locate the import location
                var rgImport = new RegExp("import\\s+{[\\s,\\w]*" + escapeStringRegexp(cls) + "[\\s,\\w]*}\\s+from\\s+['\"]([^'\"]+)['\"]", 'g');
                return rxDoc.pipe(operators_1.mergeMap(function (doc) {
                    var imp = rgImport.exec(doc);
                    if (!!imp && imp.length > 1) {
                        // extract the path
                        var relPath = imp[1].trim();
                        if (_isRelative(relPath)) {
                            // resolve against the current path
                            var absImport = path_1.normalize(path_1.join(parsedPath.dir, relPath + '.jsx'));
                            var relImport = './' +
                                _stripSuffix(lodash_1.replace(path_1.relative(aConfig.root, absImport), /\\/g, '/'));
                            // recurse
                            return _doExtractBaseClass({
                                root: aConfig.root,
                                path: relImport,
                                class: cls,
                                abs: absImport
                            }, aCycle).pipe(operators_1.distinct(_distinctClassDescriptor));
                        }
                    }
                    // nothing
                    return rxjs_1.empty();
                }));
            }));
            // done
            return rxBaseClasses.pipe(operators_1.startWith(aConfig));
        }
        /*----------------------------------------------------*/
        // log this
        aLogger.next("_createOverview ...");
        // read all jsx files
        // only look in aSource/src/LAYOUTS_PATH
        var jsxSource = aSource.pipe(operators_1.map(function (src) { return path_1.join(src, 'src', LAYOUTS_PATH); }));
        var jsxFiles = jsxSource.pipe(operators_1.tap(function (src) { return aLogger.next("_createOverview [src=" + src + "]."); }), operators_1.mergeMap(rx_walk_1.walkFiles), operators_1.filter(function (f) { return f.stats.isFile(); }), operators_1.distinct(function (f) { return f.absPath; }), operators_1.filter(function (f) { return f.absPath.endsWith('.jsx'); }));
        // read the class names of these files
        var rxClasses = jsxFiles.pipe(operators_1.mergeMap(_extractClass), operators_1.tap(function (c) { return aLogger.next("Class [" + c.class + "], [" + c.path + "]."); }), operators_1.share());
        // read the layouts
        var rxSortedClasses = rxClasses.pipe(operators_1.toArray(), 
        // order the imports
        operators_1.map(function (a) { return a.sort(function (l, r) { return l.class.localeCompare(r.class); }); }), operators_1.tap(lodash_1.noop, undefined, function () { return aLogger.next("files done."); }), operators_1.shareReplay());
        // imports
        var baseCycles = {};
        var rxSortedImports = rxClasses.pipe(operators_1.mergeMap(function (cls) { return _doExtractBaseClass(cls, baseCycles); }), operators_1.distinct(_distinctClassDescriptor), operators_1.toArray(), 
        // order the imports
        operators_1.map(function (a) { return a.sort(function (l, r) { return l.class.localeCompare(r.class); }); }), operators_1.tap(lodash_1.noop, undefined, function () { return aLogger.next("base classes done."); }), operators_1.shareReplay());
        // overview and export template generation + saving
        var overviewTemplate = aTemplate.appRegisterComponents;
        var overviewRes = rxjs_1.combineLatest(overviewTemplate, rxSortedClasses).pipe(operators_1.map(function (_a) {
            var tmp = _a[0], ctx = _a[1];
            return tmp(ctx);
        }));
        var REGISTRATION_FILE = "registration.js";
        var overviewFile = aSource.pipe(operators_1.map(function (src) { return path_1.join(src, 'src', REGISTRATION_FILE); }));
        var overviewWritten = rxjs_1.combineLatest(overviewFile, overviewRes).pipe(operators_1.mergeMap(function (_a) {
            var dst = _a[0], data = _a[1];
            return rx_file_1.writeTextFileSafe(dst, data, true);
        }));
        var exportTemplate = aTemplate.indexExports;
        var exportRes = rxjs_1.combineLatest(exportTemplate, rxSortedImports).pipe(operators_1.map(function (_a) {
            var tmp = _a[0], ctx = _a[1];
            return tmp(ctx);
        }));
        var EXPORT_FILE = "index.js"; // TODO: make this layouts/index.js
        var exportsFile = aSource.pipe(operators_1.map(function (src) { return path_1.join(src, 'src', LAYOUTS_PATH, EXPORT_FILE); }));
        var exportsWritten = rxjs_1.combineLatest(exportsFile, exportRes).pipe(operators_1.mergeMap(function (_a) {
            var dst = _a[0], data = _a[1];
            return rx_file_1.writeTextFileSafe(dst, data, true);
        }));
        // total files
        var total = [overviewWritten, exportsWritten];
        return rxjs_1.merge.apply(void 0, total).pipe(operators_1.tap(lodash_1.noop, undefined, function () { return aLogger.next('createOverview done.'); }));
    });
}
// Actual command class
// does the setup by making templates, matcher, tools, registries
// calls _createLayouts() -- does everything else
var CreateReactCommand = /** @class */ (function (_super) {
    __extends(CreateReactCommand, _super);
    function CreateReactCommand(options) {
        var _this = _super.call(this, options) || this;
        _this.options = options;
        return _this;
    }
    CreateReactCommand.prototype.printHelp = function () {
        // log the command line
        console.log(CreateReactCommand.COMMAND, CreateReactCommand.TYPE, "--" + constants_1.OPTION_SOURCE + " <SOURCE>", "--" + constants_1.OPTION_DATA + " <DATA>", "--" + constants_1.OPTION_MODULE + " <MODULE>", "[--" + constants_1.OPTION_MODULES + "]", "[--" + constants_1.OPTION_FLAT + "]", "[--" + constants_1.OPTION_SCSS + "]", "[--" + constants_1.OPTION_EDITABLE + "]", "[--" + constants_1.OPTION_VERBOSE + "]");
        // log the parameters
        console.log("--" + constants_1.OPTION_SOURCE + " <SOURCE>", ':', help_1.HELP_COMPONENT_SOURCE);
        console.log("--" + constants_1.OPTION_DATA + " <DATA>", ':', help_1.HELP_DATA_DIRECTORY);
        console.log("--" + constants_1.OPTION_MODULE + " <MODULE>", ':', help_1.HELP_MODULE_FILE);
        console.log("--" + constants_1.OPTION_MODULES, ':', help_1.HELP_MODULES);
        console.log("--" + constants_1.OPTION_FLAT, ':', help_1.HELP_FLAT);
        console.log("--" + constants_1.OPTION_SCSS, ':', help_1.HELP_SCSS);
        console.log("--" + constants_1.OPTION_EDITABLE, ':', help_1.HELP_EDITABLE);
        console.log("--" + constants_1.OPTION_VERBOSE, ':', help_1.HELP_VERBOSE);
    };
    CreateReactCommand.prototype.process = function () {
        var _this = this;
        return rxjs_1.defer(function () {
            // current options
            var opts = _this.options;
            // the mkdirp calls
            var mkdirp = rx_copy_1.rxMkDirs();
            // the directories
            // const srcDirOpt = optNormalizeDir('.');
            var srcDirOpt = options_1.optNormalizeDir(file_utils_1.firstOf(opts.src) || '.');
            console.log("OPTS.SRC: ", opts.src);
            console.log("SRC DIR OPT: ", srcDirOpt);
            // fallback source directories
            // const srcDirs = (['.']).map(optNormalizeDir);
            var srcDirs = (file_utils_1.arrayOf(opts.src) || [srcDirOpt]).map(options_1.optNormalizeDir);
            console.log("SRC DIRS: ", srcDirs);
            var rxSrcDir = mkdirp(srcDirOpt);
            var rxSrcDirs = rxjs_1.concat.apply(void 0, srcDirs.map(function (dir) { return mkdirp(dir); })).pipe(operators_1.toArray(), operators_1.shareReplay(1));
            var logger = _this.getLogger();
            var rxWchTools = _this.getWchTools();
            // Compile the templates
            // the one-line additions (for each type) for registering the component and exporting
            var rxAppRegisterComponents = rx_hbs_1.hbsCompile(path_1.join(ASSETS_BASE, 'react.registration.hbs'));
            var rxIndexExports = rx_hbs_1.hbsCompile(path_1.join(ASSETS_BASE, 'react.index.hbs'));
            // jsx and scss templates
            var rxLayoutJsx = rx_hbs_1.hbsCompile(path_1.join(ASSETS_BASE, 'react.jsx.hbs'));
            var rxLayoutCss = rx_hbs_1.hbsCompile(path_1.join(ASSETS_BASE, 'react.scss.hbs'));
            // package into a Templates obj
            var templates = {
                appRegisterComponents: rxAppRegisterComponents,
                indexExports: rxIndexExports,
                layoutJsx: rxLayoutJsx,
                layoutCss: rxLayoutCss,
            };
            // matcher with type checking
            var matcher = wchtools_1.wchToolsGetMatcher(_this.options);
            var typeMatcher = function (aType) { return aType && aType.name && matcher(aType.name); };
            // the registries
            var layoutReg = new LayoutRegistry(rxSrcDirs, opts, mkdirp, logger);
            // attach to the tools
            return rxWchTools.pipe(operators_1.mergeMap(function (aWchTools) {
                return _createLayouts(aWchTools, rxSrcDir, templates, typeMatcher, opts, layoutReg, logger);
            }), operators_1.distinct());
        });
    };
    CreateReactCommand.TYPE = 'react_components';
    CreateReactCommand.COMMAND = constants_1.CMD_CREATE;
    return CreateReactCommand;
}(abstract_command_1.AbstractCommand));
exports.CreateReactCommand = CreateReactCommand;
var lodash_2 = require("lodash");
var number_to_words_1 = require("number-to-words");
var operators_2 = require("rxjs/operators");
function _getStyleExtension(aOptions) {
    return aOptions.scss ? '.scss' : '.css';
}
function _kebabCase(aValue) {
    return transliteration_1.kebabCase(aValue);
}
var LEADING_DIGIT = /^(\d*)/g;
var LAYOUTS_PATH = 'layouts';
var LAYOUT_SUFFIX = 'Layout';
function _createLayoutComponentName(aName) {
    // trim
    var trimmedName = aName.trim();
    var fixed = trimmedName.replace(LEADING_DIGIT, function (m, p) { return (p && (p.length > 0)) ? number_to_words_1.toWords(p) + ' ' : ''; });
    /** check if the layout component starts with a number, in this case
     * convert the number to a string */
    var name = lodash_1.upperFirst(_camelCase(fixed));
    // remove 'Layout' from the end of component name
    if (name.endsWith(LAYOUT_SUFFIX)) {
        name = name.substr(0, name.length - LAYOUT_SUFFIX.length);
    }
    return name;
}
function _getLayoutPath(aLayout, aType, aOptions) {
    return LAYOUTS_PATH;
    // function _getPathForLayout(aName: string, aType: any, aOptions: Options): string {
    //     // name of the layout folder segment
    //     let layoutName = _createLayoutComponentName(aName);
    //     // layout folder
    //     const layoutFolder = _kebabCase(layoutName);
    //     // access the type properties
    //     const tags: string[] = aType.tags || [];
    //     // convert tags into segments
    //     const segments = aOptions.flat ? [] : tags.map(_kebabCase).sort();
    //     segments.push(layoutFolder);
    //     // return this path
    //     return segments.join(sep);
    // }
    // console.log("stuff:", _getPathForLayout(aLayout.name, aType, aOptions));
    // return join(LAYOUTS_PATH, _getPathForLayout(aLayout.name, aType, aOptions));
}
/**
 * Constructs the relevant information for a components class
 *
 * @param aDir      base directory
 * @param aType     type object
 * @param aOptions  options
 * @param aLogger   logger
 *
 * @return the component object
 */
function _initLayoutClass(aDir, aType, aLayout, aOptions, aLogger) {
    // check for the path suffix
    var layoutPath = _getLayoutPath(aLayout, aType, aOptions);
    var layoutName = aLayout.name || aType.name;
    // component names
    console.log("LAYOUT NAME:", layoutName);
    var layoutBaseName = _createLayoutComponentName(layoutName);
    var layoutClass = layoutBaseName;
    // references
    var layoutRef = lodash_2.lowerFirst(layoutBaseName);
    // template and style
    var layoutTemplateFile = layoutRef + '.jsx'; // need this to be dependent on angular vs react
    var layoutStyleFile = layoutRef + _getStyleExtension(aOptions);
    // selector
    // const layoutSelector = _kebabCase(layoutClass);
    var layoutSelector = "is this even used lmao";
    // the root folder
    // const folder = normalize(join(aDir, layoutPath));
    var jsxFolder = path_1.normalize(path_1.join(aDir, 'src', layoutPath));
    var cssFolder = path_1.normalize(path_1.join(aDir, 'styles', layoutPath));
    console.log("ADIR:", aDir);
    var result = {
        type: aType, layout: aLayout, jsxFolder: jsxFolder, cssFolder: cssFolder, layoutClass: layoutClass, layoutRef: layoutRef,
        layoutTemplateFile: layoutTemplateFile, layoutStyleFile: layoutStyleFile, layoutSelector: layoutSelector, layoutTemplate: aLayout.template
    };
    // log this
    console.log("Type: " + aType.name + ", LayoutClass: " + JSON.stringify(layoutClass) + ", Jsx folder: " + jsxFolder + ", Css folder: " + cssFolder);
    // ok
    return result;
}
/**
 * Finds the directory tha the component should be created in. Tests
 * each potential source directory for an existing component file. If non exists, uses the
 * first source directory
 *
 * @param aType        type
 * @param aSources     sources directory
 *
 * @return the selected source directory
 */
function _getLayoutsDir(aLayout, aType, aSources, aOptions, aMkdirp, aLogger) {
    return aSources.pipe(operators_1.map(function (sources) { return sources[0]; }), operators_1.mergeMap(aMkdirp));
    // // check for the path suffix
    // const typePath = _getLayoutPath(aLayout, aType, aOptions);
    // const className = _createLayoutComponentName(aLayout.name);
    // const relPath = join(typePath, _camelCase(className) + '.ts');
    // // map each source to a check
    // return rxFindDir(aSources, relPath, aMkdirp).pipe(
    //     tap(dir => console.log(`Selected layout source directory [${dir}] for layout [${className}].`))
    // );
}
function _createLayoutClass(aLayout, aType, aSources, aOptions, aMkdirp, aLogger) {
    // the type
    var type = aType;
    // locate the component directory
    return _getLayoutsDir(aLayout, type, aSources, aOptions, aMkdirp, aLogger).pipe(operators_2.first(), operators_1.map(function (dir) { return _initLayoutClass(dir, aType, aLayout, aOptions, aLogger); }), operators_1.mergeMap(function (clz) { return rxjs_1.concat(aMkdirp(clz.jsxFolder), aMkdirp(clz.cssFolder)).pipe(operators_1.mapTo(clz)); }), operators_1.shareReplay());
}
var LayoutRegistry = /** @class */ (function () {
    function LayoutRegistry(aBaseFolders, aOptions, aMkdirp, aLogger) {
        this.aBaseFolders = aBaseFolders;
        this.aOptions = aOptions;
        this.aMkdirp = aMkdirp;
        this.aLogger = aLogger;
        this.layoutMap = {
            byId: {},
            byName: {},
            dirById: {}
        };
        this.layoutRegistry = {};
    }
    LayoutRegistry.prototype.registerLayout = function (aLayout) {
        var layout = aLayout;
        var lid = layout.id;
        var lname = layout.name;
        if (this.layoutMap.byId[lid] || this.layoutMap.byName[lname]) {
            // do not create a layout, twice
            return false;
        }
        // register
        this.layoutMap.byId[lid] = layout;
        this.layoutMap.byName[lname] = layout;
        // ok
        return true;
    };
    LayoutRegistry.prototype.findLayoutClass = function (aLayout, aType) {
        // the name
        var layoutId = aLayout.id;
        // check if we are already searching
        var rxRunning = this.layoutRegistry[layoutId];
        if (!rxRunning) {
            // try to find the folder
            rxRunning = this.layoutRegistry[layoutId] =
                _createLayoutClass(aLayout, aType, this.aBaseFolders, this.aOptions, this.aMkdirp, this.aLogger);
        }
        // ok
        return rxRunning;
    };
    return LayoutRegistry;
}());
exports.LayoutRegistry = LayoutRegistry;
