"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
/* Copyright IBM Corp. 2017 */
var crypto_1 = require("crypto");
var graceful_fs_1 = require("graceful-fs");
var lodash_1 = require("lodash");
var path_1 = require("path");
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var constants_1 = require("./../constants");
var abstract_command_1 = require("./abstract.command");
var nodeMap = {};
var colorMap = {};
var nodeCounter = 0;
function _getColorValue() {
    return Math.floor(Math.random() * 256);
}
function _getTypeColor(aID) {
    // some random value
    var c = colorMap[aID];
    if (!c) {
        c = colorMap[aID] = '#' + crypto_1.createHash('sha256').update(aID).digest('hex').substr(0, 6);
    }
    return c;
}
function _getNodeName(aID) {
    // test
    var name = nodeMap[aID];
    if (!name) {
        name = nodeMap[aID] = "node" + nodeCounter++;
    }
    return name;
}
function _getLabel(aString) {
    return lodash_1.escape(aString).replace(/\s/g, '\\n');
}
var SITE_TYPE = 'site';
function _writeContent(aContent, aTools, aOut) {
    // write the content node
    var id = aContent.id;
    var name = _getNodeName(id);
    // elements
    var elements = aContent.elements;
    // write the site node
    aOut.write(name + "[tooltip=\"" + id + "\";label=\"" + _getLabel(aContent.name) + " | {");
    // write the elements
    if (elements) {
        // write the node name
        Object.keys(elements).forEach(function (key, idx) {
            // name the element
            var nodeID = _getNodeName(id + "#" + key + ")");
            // write
            var value = elements[key];
            var sep = (idx > 0) ? '|' : '';
            aOut.write(sep + "<" + nodeID + ">" + lodash_1.escape(key));
        });
    }
    // write
    aOut.write("}\", color=\"" + _getTypeColor(aContent.typeId) + "\"];\n");
    // write outgoing links
    if (elements) {
        // write the node name
        Object.keys(elements).forEach(function (key, idx) {
            // name the element
            var nodeID = _getNodeName(id + "#" + key + ")");
            var value = elements[key];
            // check if this is a reference
            if (value.elementType === 'reference') {
                // handle the values
                if (value.values && lodash_1.isArray(value.values)) {
                    // iterate over the values
                    value.values.forEach(function (dstLink) {
                        // write the link
                        var dstName = _getNodeName(dstLink.id);
                        aOut.write("\"" + name + "\":" + nodeID + " -> \"" + dstName + "\";\n");
                    });
                }
                // handle a single value
                if (value.value && value.value.id && lodash_1.isString(value.value.id)) {
                    // write the link
                    var dstName = _getNodeName(value.value.id);
                    aOut.write("\"" + name + "\":" + nodeID + " -> \"" + dstName + "\";\n");
                }
            }
        });
    }
}
function _writeSite(aSite, aTools, aOut) {
    // write the site
    var id = aSite.data.id;
    var name = _getNodeName(id);
    // write the site node
    aOut.write(name + "[tooltip=\"" + id + "\";label=\"" + _getLabel(aSite.data.name) + "\", shape=oval, color=\"" + _getTypeColor(aSite.data.classification) + "\"];\n");
    // write the actual nodes
    if (aSite.children) {
        // iterate over the children
        aSite.children.forEach(function (site) {
            // write the site
            _writeSite(site, aTools, aOut);
            // writes the page links
            var dstName = _getNodeName(site.data.id);
            aOut.write("\"" + name + "\" -> \"" + dstName + "\";\n");
        });
    }
    // check of we have content
    var contentId = aSite.data.contentId;
    if (contentId) {
        // write the link
        var dstName = _getNodeName(contentId);
        aOut.write("\"" + name + "\" -> \"" + dstName + "\";\n");
    }
}
function _writeDigraph(aTools, aOut) {
    // produce the digraph
    aOut.write('digraph g {\n');
    aOut.write("ranksep = \"1.0 equally\";\n");
    // shape config
    aOut.write('node [shape = record];\n');
    // write all sites
    var sites = aTools.sites;
    // tslint:disable-next-line:forin
    for (var siteId in sites) {
        _writeSite(sites[siteId], aTools, aOut);
    }
    // write all content
    var content = aTools.content;
    // tslint:disable-next-line:forin
    for (var contentId in content) {
        _writeContent(content[contentId], aTools, aOut);
    }
    // ok
    aOut.write('}\n');
}
function _showGraph(aOutFile, aWchTools) {
    // normalize the target
    var outFile = path_1.normalize(aOutFile);
    // output stream
    var wrt = graceful_fs_1.createWriteStream(outFile);
    // write
    _writeDigraph(aWchTools, wrt);
    // target
    return rxjs_1.of(aOutFile);
}
/**
 * show graph --data <DATA> --out <TARGETNAME>
 */
var ShowGraphCommand = /** @class */ (function (_super) {
    __extends(ShowGraphCommand, _super);
    function ShowGraphCommand(options) {
        var _this = _super.call(this, options) || this;
        _this.options = options;
        return _this;
    }
    ShowGraphCommand.prototype.process = function () {
        // access to the data folders
        var opts = this.options;
        // out
        var out = path_1.resolve(opts.out || './graph.dot');
        // process
        var rxWchTools = this.getWchTools();
        // process
        var rxGraph = rxWchTools.pipe(operators_1.mergeMap(function (aTools) { return _showGraph(out, aTools); }));
        // ok
        return rxGraph;
    };
    ShowGraphCommand.TYPE = 'graph';
    ShowGraphCommand.COMMAND = constants_1.CMD_SHOW;
    return ShowGraphCommand;
}(abstract_command_1.AbstractCommand));
exports.ShowGraphCommand = ShowGraphCommand;
