"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Assert = require("assert");
var valid_url_1 = require("valid-url");
var email_validator_1 = require("email-validator");
var lodash_1 = require("lodash");
var url_utils_1 = require("./url.utils");
function assertNotNull(aValue, aName) {
    Assert.ok(aValue != null, aName ? "Value [" + aName + "] must not be null or undefined." : 'Value must not be null or undefined.');
}
exports.assertNotNull = assertNotNull;
function assertParameter(aValue, aParameterName) {
    Assert.ok(!!aValue, "Please specify the '--" + aParameterName + "' parameter.");
}
exports.assertParameter = assertParameter;
function isValidUrl(aValue) {
    return lodash_1.isString(aValue) && !!valid_url_1.isWebUri(aValue);
}
exports.isValidUrl = isValidUrl;
function assertIsUrl(aValue, aName) {
    Assert.ok(isValidUrl(aValue), aName ? "Value [" + aName + "] must be a valid URL." : 'Value must be a valid URL.');
    return aValue;
}
exports.assertIsUrl = assertIsUrl;
function isValidEmail(aValue) {
    return lodash_1.isString(aValue) && !!email_validator_1.validate(aValue);
}
exports.isValidEmail = isValidEmail;
function isValidUserName(aValue) {
    return lodash_1.isString(aValue) && (!!email_validator_1.validate(aValue) || (aValue === 'apikey'));
}
exports.isValidUserName = isValidUserName;
function assertIsEmail(aValue, aName) {
    Assert.ok(isValidEmail(aValue), aName ? "Value [" + aName + "] must be a valid e-mail address." : 'Value must be a valid e-mail address.');
    return aValue;
}
exports.assertIsEmail = assertIsEmail;
function assertIsValidUserName(aValue, aName) {
    Assert.ok(isValidUserName(aValue), aName
        ? "Value [" + aName + "] must be a valid e-mail address or the term 'apikey'."
        : 'Value must be a valid e-mail address or the term \'apikey\'.');
    return aValue;
}
exports.assertIsValidUserName = assertIsValidUserName;
function assertHasTrailingSlash(aValue) {
    Assert.ok(isValidUrl(aValue) && url_utils_1.hasTrailingSlash(aValue), 'URL [${aValue}] must end with a slash.');
    return aValue;
}
exports.assertHasTrailingSlash = assertHasTrailingSlash;
