"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var lodash_1 = require("lodash");
var graph_1 = require("./graph");
function getTypeEdges(aWchTools) {
    // the set of edges
    var edges = new Set();
    var cache = new Map();
    var allTypes = lodash_1.values(aWchTools.types);
    function _handleElements(aElements) {
        // iterate
        lodash_1.forIn(aElements, function (element) {
            // check the type
            var type = element.elementType;
            if (type === 'reference') {
                // we have a type restriction
                if (lodash_1.isArray(element.restrictTypes)) {
                    // work on these types
                    element.restrictTypes.forEach(function (res) { return edges.add(aWchTools.types[res.id]); });
                }
                else {
                    // all types
                    allTypes.forEach(function (type) { return edges.add(type); });
                }
            }
            else if (type === 'group') {
                // recurse
                _handleElements(element);
            }
        });
    }
    function _getEdges(aType) {
        // test if we know the result
        var cachedEdges = cache.get(aType);
        if (lodash_1.isArray(cachedEdges)) {
            return cachedEdges;
        }
        // rest
        edges.clear();
        // iterate over the types
        if (aType.elements) {
            // work on the elements
            _handleElements(aType.elements);
        }
        // ok
        var res = Array.from(edges);
        cache.set(aType, res);
        // ok
        return res;
    }
    return _getEdges;
}
exports.getTypeEdges = getTypeEdges;
function _isPageType(aType) {
    return aType && lodash_1.isArray(aType.kind) && (aType.kind.indexOf('page') >= 0);
}
function getPageTypes(aWchTools) {
    return lodash_1.values(aWchTools.types).filter(_isPageType);
}
exports.getPageTypes = getPageTypes;
function getDependentTypes(aTypes, aWchTools) {
    // edges
    var edges = getTypeEdges(aWchTools);
    // resolve
    return aTypes.map(function (type) { return graph_1.connectedComponents([type], edges); });
}
exports.getDependentTypes = getDependentTypes;
function getPageTypeDependencies(aWchTools) {
    return getDependentTypes(getPageTypes(aWchTools), aWchTools);
}
exports.getPageTypeDependencies = getPageTypeDependencies;
