"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
exports.__esModule = true;
/* Copyright IBM Corp. 2017 */
var crypto_1 = require("crypto");
var mime_1 = require("mime");
var path_1 = require("path");
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var uuid_1 = require("uuid");
var file_utils_1 = require("../utils/file.utils");
var constants_1 = require("./../constants");
var options_1 = require("./../options");
var json_1 = require("./../utils/json");
var revision_utils_1 = require("./../utils/revision.utils");
var rx_walk_1 = require("./../utils/rx.walk");
var abstract_command_1 = require("./abstract.command");
function _fixAssetType(aAsset, aSrc, aName) {
    // parse the filename
    var parserName = path_1.parse(aSrc);
    // generate the md5
    var rxMd5 = rx_walk_1.readFile(aSrc).pipe(operators_1.map(function (data) { return crypto_1.createHash('md5').update(data).digest('base64'); }));
    // stats
    var stats = rx_walk_1.rxStats(aSrc);
    // resource id
    var resId = uuid_1.v4();
    // add ID
    aAsset.id = uuid_1.v4();
    aAsset.name = aName;
    aAsset.mediaType = mime_1.getType(aSrc);
    // construct a path
    var damid = uuid_1.v4();
    var prefix = damid.substr(0, 2);
    // final path
    var path = "/dxdam/" + prefix + "/" + damid + "/" + aName;
    aAsset.path = path;
    // type
    aAsset.assetType = aAsset.mediaType.split('/')[0];
    aAsset.isManaged = true;
    aAsset.fileName = parserName.base;
    // resource infos
    aAsset.resource = resId;
    // renditions support
    aAsset.renditions = {
        'default': {
            'id': "r=" + resId + "&a=" + aAsset.id,
            'source': "/authoring/v1/resources/" + resId
        }
    };
    // add stats
    return stats.pipe(operators_1.map(function (s) { return aAsset.fileSize = s.size; }), operators_1.mergeMap(function () { return rxMd5.pipe(operators_1.map(function (digest) {
        aAsset.digest = digest;
        return json_1.canonicalizeJSON(revision_utils_1.addRevisionId(aAsset));
    })); }));
}
function _createAssetDoc(aSrc, aName) {
    // load the template
    return rx_walk_1.readFile(path_1.join(__dirname, '..', '..', '..', 'assets', 'asset.json')).pipe(operators_1.map(function (data) { return JSON.parse(data); }), operators_1.mergeMap(function (template) { return _fixAssetType(template, aSrc, aName); }));
}
function _writeAsset(aSrc, aName, aDataDir, aLogger) {
    // construct the object
    return _createAssetDoc(aSrc, aName).pipe(operators_1.mergeMap(function (doc) {
        // get the path
        var path = doc.path;
        var parsedPath = path_1.parse(path);
        // target dir
        var dirAsset = path_1.join(aDataDir, 'assets', parsedPath.dir);
        var dir = rx_walk_1.rxMkdirp(dirAsset);
        // copy
        var fileCopy = rx_walk_1.rxCopyFile(aSrc, path_1.join(aDataDir, 'assets', path));
        // write the file
        var filename = dirAsset + "/" + aName + "_amd.json";
        var jsonFile = dir.pipe(operators_1.mergeMap(function (d) { return rx_walk_1.writeFile(filename, JSON.stringify(doc, undefined, 2)); }));
        // merge
        return rxjs_1.merge(fileCopy, jsonFile);
    }));
}
exports.writeAsset = _writeAsset;
/**
 * Generates angular components based on type and layout mapping definitions
 *
 * create assert --data DATA_FOLDER --src ASSET_FILE
 */
var CreateAssetCommand = /** @class */ (function (_super) {
    __extends(CreateAssetCommand, _super);
    function CreateAssetCommand(options) {
        var _this = _super.call(this, options) || this;
        _this.options = options;
        return _this;
    }
    CreateAssetCommand.prototype.process = function () {
        // logger
        var logger = this.getLogger();
        // data directory
        var opts = this.options;
        var dataDirOpt = this.getDataDir();
        var rxDataDir = this.rxMkdirp(dataDirOpt);
        // source
        var src = options_1.optNormalizeDir(file_utils_1.firstOf(opts.src) || 'undefined.jpg');
        // parse
        var parsedSrc = path_1.parse(src);
        var name = opts.name || parsedSrc.base;
        // log this
        logger.next("Data dir [" + dataDirOpt + "], source file [" + src + "].");
        // load wchtools
        return rxDataDir.pipe(operators_1.mergeMap(function (aDataDir) { return _writeAsset(src, name, aDataDir, logger); }));
    };
    CreateAssetCommand.TYPE = 'asset';
    CreateAssetCommand.COMMAND = constants_1.CMD_CREATE;
    return CreateAssetCommand;
}(abstract_command_1.AbstractCommand));
exports.CreateAssetCommand = CreateAssetCommand;
