"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
exports.__esModule = true;
/* Copyright IBM Corp. 2017 */
var lodash_1 = require("lodash");
var path_1 = require("path");
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var constants_1 = require("./../constants");
var assert_1 = require("./../utils/assert");
var json_1 = require("./../utils/json");
var rx_file_1 = require("./../utils/rx.file");
var rx_walk_1 = require("./../utils/rx.walk");
var abstract_command_1 = require("./abstract.command");
var help_1 = require("./help");
function _byPosition(aLeft, aRight) {
    var leftPos = (aLeft && aLeft.position) ? aLeft.position : Number.MAX_SAFE_INTEGER, rightPos = (aRight && aRight.position) ? aRight.position : Number.MAX_SAFE_INTEGER;
    // compare
    return (leftPos < rightPos) ? -1 : (leftPos > rightPos) ? +1 : 0;
}
function _buildSiteChildren(aRootPath, aChildren) {
    var result = aChildren ? aChildren.map(function (node) {
        var res = lodash_1.cloneDeep(node.data);
        // build the route
        var route = aRootPath + encodeURIComponent(res.segment) + '/';
        res.route = route;
        // recurse
        res.children = _buildSiteChildren(route, node.children);
        return res;
    }) : [];
    // sort
    return result.sort(_byPosition);
}
function _writeSite(aSiteName, aAppAssetDir, aSite, aLogger) {
    // some assertions
    assert_1.assertNotNull(aSiteName, 'siteName');
    assert_1.assertNotNull(aAppAssetDir, 'assetDir');
    assert_1.assertNotNull(aSite, 'site');
    assert_1.assertNotNull(aLogger, 'logger');
    // build the target folder
    var dirName = path_1.join(aAppAssetDir, 'delivery', 'v1', 'rendering', 'sites');
    var fileName = path_1.join(dirName, aSiteName + '.json');
    // log this
    aLogger.next("Writing site feed to [" + fileName + "] ...");
    // construct the target directory
    return rx_walk_1.rxMkdirp(dirName).pipe(operators_1.mergeMap(function () { return rx_file_1.writeJsonFileSafe(fileName, aSite, true); }));
}
function _buildSite(aSiteName, aTools) {
    // the result
    var result = {};
    // access the site
    var root = aTools.sites[aSiteName];
    if (root) {
        // start with the root
        result = lodash_1.cloneDeep(root.data);
        result.pages = _buildSiteChildren('/', root.children);
    }
    // done
    return rxjs_1.of(json_1.canonicalizeJSON(result));
}
/**
 * Generates angular components based on type and layout mapping definitions
 *
 * create site --data DATA_FOLDER --name APP_NAME
 */
var CreateSiteCommand = /** @class */ (function (_super) {
    __extends(CreateSiteCommand, _super);
    function CreateSiteCommand(options) {
        var _this = _super.call(this, options) || this;
        _this.options = options;
        return _this;
    }
    CreateSiteCommand.prototype.printHelp = function () {
        // log the command line
        console.log(CreateSiteCommand.COMMAND, CreateSiteCommand.TYPE, "--" + constants_1.OPTION_NAME + " <NAME>", "[--" + constants_1.OPTION_DATA + " <DATA>]", "[--" + constants_1.OPTION_VERBOSE + "]");
        // log the parameters
        console.log("--" + constants_1.OPTION_NAME, ':', help_1.HELP_APPLICATION_NAME);
        console.log("--" + constants_1.OPTION_DATA, ':', help_1.HELP_DATA_DIRECTORY);
        console.log("--" + constants_1.OPTION_VERBOSE, ':', help_1.HELP_VERBOSE);
    };
    CreateSiteCommand.prototype.process = function () {
        // logger
        var logger = this.getLogger();
        // data directory
        // current options
        var opts = this.options;
        // site name
        var siteName = 'default';
        // the directories
        var appName = opts.name || this.getAppName();
        var appAssetDir = this.getAppAssetDir(appName);
        // the tools
        var rxWchTools = this.getWchTools();
        // the site
        var rxSite = rxWchTools.pipe(operators_1.mergeMap(function (aTools) { return _buildSite(siteName, aTools); }), operators_1.mergeMap(function (aSite) { return _writeSite(siteName, appAssetDir, aSite, logger); }));
        // ok
        return rxSite;
    };
    CreateSiteCommand.TYPE = 'site';
    CreateSiteCommand.COMMAND = constants_1.CMD_CREATE;
    return CreateSiteCommand;
}(abstract_command_1.AbstractCommand));
exports.CreateSiteCommand = CreateSiteCommand;
