"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
exports.__esModule = true;
/* Copyright IBM Corp. 2017 */
var path_1 = require("path");
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var constants_1 = require("../constants");
var file_utils_1 = require("../utils/file.utils");
var options_1 = require("./../options");
var abstract_command_1 = require("./abstract.command");
var create_components_command_1 = require("./create.components.command");
var help_1 = require("./help");
/**
 * Generates angular components based on type and layout mapping definitions
 *
 * register layoutcomponents --src SOURCE_CODE_FOLDER [--module MODULENAME] [--verbose]
 */
var RegisterLayoutComponentsCommand = /** @class */ (function (_super) {
    __extends(RegisterLayoutComponentsCommand, _super);
    function RegisterLayoutComponentsCommand(options) {
        var _this = _super.call(this, options) || this;
        _this.options = options;
        return _this;
    }
    RegisterLayoutComponentsCommand.prototype.printHelp = function () {
        // log the command line
        console.log(RegisterLayoutComponentsCommand.COMMAND, RegisterLayoutComponentsCommand.TYPE, "--" + constants_1.OPTION_SOURCE + " <SRC>", "[--" + constants_1.OPTION_MODULE + " <MODULE_FILE>]", "[--" + constants_1.OPTION_SKIP_REGISTRATION + "]", "[--" + constants_1.OPTION_VERBOSE);
        // log the parameters
        console.log("--" + constants_1.OPTION_SOURCE + " <SRC>", ':', help_1.HELP_COMPONENT_SOURCE);
        console.log("--" + constants_1.OPTION_MODULE + " <MODULE_FILE>", ':', help_1.HELP_MODULE_FILE);
        console.log("--" + constants_1.OPTION_SKIP_REGISTRATION, ':', help_1.HELP_SKIP_REGISTRATION);
        console.log("--" + constants_1.OPTION_VERBOSE, ':', help_1.HELP_VERBOSE);
    };
    RegisterLayoutComponentsCommand.prototype.process = function () {
        // current options
        var opts = this.options;
        var srcDirOpt = options_1.optNormalizeDir(file_utils_1.firstOf(opts.src) || path_1.join(this.getAppDir(), 'app'));
        var rxSrcDir = this.rxMkdirp(srcDirOpt);
        // module file
        var moduleFile = opts.module || path_1.join(srcDirOpt, constants_1.APP_MODULE_FILE);
        var layoutsFile = path_1.join(srcDirOpt, constants_1.LAYOUTS_FILE);
        // logger
        var logger = this.getLogger();
        // the compiler template
        var templates = create_components_command_1.createComponentTemplates(opts, logger);
        // log the name
        logger.next("Source dir [" + srcDirOpt + "], ngModule [" + moduleFile + "], layouts [" + layoutsFile + "].");
        // constructs the overview files
        var overview = create_components_command_1.createComponentOverview(rxSrcDir, templates, opts, logger);
        // registers the overview with the application
        var rewrite = create_components_command_1.rewriteAppModule(layoutsFile, moduleFile, opts, logger);
        // merge
        return rxjs_1.merge(overview, rewrite).pipe(operators_1.distinct());
    };
    RegisterLayoutComponentsCommand.TYPE = 'layoutcomponents';
    RegisterLayoutComponentsCommand.COMMAND = constants_1.CMD_REGISTER;
    return RegisterLayoutComponentsCommand;
}(abstract_command_1.AbstractCommand));
exports.RegisterLayoutComponentsCommand = RegisterLayoutComponentsCommand;
