"use strict";
exports.__esModule = true;
/* Copyright IBM Corp. 2018 */
var graceful_fs_1 = require("graceful-fs");
var path_1 = require("path");
var request = require("request");
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var unzip_1 = require("unzip");
var rx_walk_1 = require("./rx.walk");
var random_ua = require('random-ua');
function _skipPrefix(aName, aCount) {
    // current name
    var idx = 0;
    for (var i = 0; i < aCount; ++i) {
        // find the next separator
        var nextIdx = aName.indexOf('/', idx);
        if (nextIdx >= idx) {
            idx = nextIdx + 1;
        }
        else {
            return null;
        }
    }
    // split
    return aName.substring(idx);
}
function _rxExtractEntry(aEntry, aDstDir, aSkip) {
    // skip the prefix
    var path = _skipPrefix(aEntry.path, aSkip);
    if (!path) {
        // nothing
        return rxjs_1.of('').pipe(operators_1.tap(function () { return aEntry.autodrain(); }), operators_1.filter(function () { return false; }));
    }
    // create filename
    var fileName = path_1.normalize(path_1.join(aDstDir, path));
    // handle directories
    if (aEntry.type === 'Directory') {
        // create the directory
        return rx_walk_1.rxMkdirp(fileName).pipe(operators_1.tap(function () { return aEntry.autodrain(); }), operators_1.filter(function () { return false; }));
    }
    else {
        // parent path
        var parsedFileName = path_1.parse(fileName);
        var dirName = parsedFileName.dir;
        // create later
        return rx_walk_1.rxMkdirp(dirName).pipe(operators_1.mergeMap(function () {
            // construct the stream
            var stream = aEntry.pipe(graceful_fs_1.createWriteStream(fileName));
            // attach
            return rxjs_1.fromEvent(stream, 'close').pipe(operators_1.first());
        }), 
        // map to the target name
        operators_1.mapTo(fileName));
    }
}
function _rxUnzipFromUrl(aSrcUrl, aDstDir, aSkip) {
    if (aSkip === void 0) { aSkip = 0; }
    // the options
    var options = {
        url: aSrcUrl,
        headers: {
            'User-Agent': random_ua.generate()
        }
    };
    // defer
    return rxjs_1.defer(function () {
        // construct the stream
        var stream = request(options).pipe(unzip_1.Parse());
        // handle
        var onEntry = rxjs_1.fromEvent(stream, 'entry');
        var onClose = rxjs_1.fromEvent(stream, 'close');
        // return the full stream
        return onEntry.pipe(operators_1.takeUntil(onClose), operators_1.mergeMap(function (entry) { return _rxExtractEntry(entry, aDstDir, aSkip); }));
    });
}
exports.rxUnzipFromUrl = _rxUnzipFromUrl;
